//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                               MACD_remake.mq5 |
//|                                                                                                                                                           hoz |
//|                                                                                                                                                               |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
#property copyright   "2009-2017, MetaQuotes Software Corp."
#property link        "http://www.mql5.com"
#property description "Moving Average Convergence/Divergence"
#include <MovingAverages.mqh>
//--- indicator settings
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_plots   2
#property indicator_type1   DRAW_HISTOGRAM
#property indicator_type2   DRAW_LINE
#property indicator_color1  Silver
#property indicator_color2  Red
#property indicator_width1  2
#property indicator_width2  1
#property indicator_label1  "MACD"
#property indicator_label2  "Signal"
//--- input parameters
input int                i_fastEMA = 12;                  // Fast EMA period
input int                i_slowEMA = 26;                  // Slow EMA period
input int                i_signalSMA = 9;                 // Signal SMA period
input ENUM_APPLIED_PRICE i_appliedPrice = PRICE_CLOSE;    // Applied price
//--- indicator buffers
double                   macdBasicBuf[];
double                   macdSignalBuf[];
double                   fastMABuf[];
double                   slowMABuf[];
//--- MA handles
int                      fastMAHandle;
int                      slowMAHandle;
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                            Custom indicator initialization function                                                           |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
void OnInit() {
//--- indicator buffers mapping
  SetIndexBuffer(0, macdBasicBuf, INDICATOR_DATA);
  SetIndexBuffer(1, macdSignalBuf, INDICATOR_DATA);
  SetIndexBuffer(2, fastMABuf, INDICATOR_CALCULATIONS);
  SetIndexBuffer(3, slowMABuf, INDICATOR_CALCULATIONS);
//--- sets first bar from what index will be drawn
  PlotIndexSetInteger(1, PLOT_DRAW_BEGIN, i_signalSMA - 1);
//--- name for Dindicator subwindow label
  IndicatorSetString(INDICATOR_SHORTNAME, "MACD_remake(" + string(i_fastEMA) + "," + string(i_slowEMA) + "," + string(i_signalSMA) + ")");
//--- get MA handles
  fastMAHandle = iMA(NULL, 0, i_fastEMA, 0, MODE_EMA, i_appliedPrice);
  slowMAHandle = iMA(NULL, 0, i_slowEMA, 0, MODE_EMA, i_appliedPrice);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                     Заполнение значениями буферов быстрой и медленноq МА                                                      |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool bothMABuffersCopying(int limit) {
//---
  while (!IsStopped()) {
    if (CopyBuffer(fastMAHandle, 0, 0, limit, fastMABuf) <= 0) {
      Print("Copying fast EMA's buffer failed! Error = ", GetLastError());
        return false;
    }
    if (CopyBuffer(slowMAHandle, 0, 0, limit, slowMABuf) <= 0) {
      Print("Copying slow SMA buffer failed! Error = ", GetLastError());
        return false;
    }
    return true;
  }
  return true;
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                            Определение индекса бара, с которого необходимо производить перерасчет                                             |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
int recalcIndex(const int barsCount,        // Количество пришедших баров таймфрейма открытого графика
                const int countedBars) {    // Количество уже обработанных баров таймфрейма открытого графика
//---
  if (countedBars != 0)
    return barsCount - countedBars;
//---
  return barsCount - 2;
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                          Custom indicator iteration function                                                                  |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[]) {
//---
  if (rates_total < i_signalSMA || i_fastEMA > i_slowEMA)
    return 0;

  int calculated = BarsCalculated(fastMAHandle);
  if (calculated < rates_total) {
    Print("Not all data of fastMAHandle is calculated (", calculated, "bars ). Error = ", GetLastError());
    return 0;
  }

  int toCopy = recalcIndex(rates_total, prev_calculated);
  if (toCopy > 0) {
    if (!bothMABuffersCopying(toCopy))
      return 0;

    int limit = recalcIndex(rates_total, prev_calculated);
//--- calculate MACD
    for (int i = limit; i > 0 && !IsStopped(); i--)
      macdBasicBuf[i] = fastMABuf[i] - slowMABuf[i];
//--- calculate Signal
    SimpleMAOnBuffer(rates_total, prev_calculated, 0, i_signalSMA, macdBasicBuf, macdSignalBuf);
  }
//---
  return rates_total;
}
